// Copyright 2014 David Eisenstat <eisenstatdavid@gmail.com>
// Released under http://opensource.org/licenses/MIT
// simplertimes August 2014 version

import java.util.Random;

public final class TestPolynomial {
    private static int[] randomPolynomial(Random r, int maxDegree) {
        int[] array = new int[r.nextInt(maxDegree + 1) + 1];
        for (int j = 0; j < array.length; j++) {
            array[j] = r.nextInt(100);
        }
        return array;
    }

    private static int[] multiplySlowly(int[] a, int[] b) {
        int[] c = new int[a.length - 1 + b.length];
        for (int i = 0; i < a.length; i++) {
            for (int j = 0; j < b.length; j++) {
                c[i + j] = Polynomial.addMultiply(c[i + j], a[i], b[j]);
            }
        }
        return c;
    }

    private static void test(Random r, int maxDegree) {
        int[] a = randomPolynomial(r, maxDegree);
        int[] b = randomPolynomial(r, maxDegree);
        int[] c = multiplySlowly(a, b);
        int[] d = Polynomial.multiply(a, b);
        if (d.length != c.length) {
            throw new RuntimeException();
        }
        for (int j = 0; j < c.length; j++) {
            if (d[j] != c[j]) {
                throw new RuntimeException();
            }
        }
    }

    public static void main(String[] args) {
        Random r;
        if (0 < args.length) {
            r = new Random(Long.parseLong(args[0]));
        } else {
            r = new Random();
        }
        for (int i = 0; i < 100000; i++) {
            test(r, 100);
        }
        for (int i = 0; i < 10; i++) {
            test(r, 10000);
        }
        System.out.println("OK");
    }
}
